/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetCompMotor.c

Abstract:

	This file contains methods for a Phidget Component Motor. 
	A component Motor can belong to several phidgets e.g. the phidget Motor Control.
	Only if one of these phidgets is attached the component is useable.	Use
	phidget_GetDeviceClass(pointerToPhidget) to call the specific phidget methods
	in phidget21 API.

--*/

#include "PhidgetCompMotor.h"

//////////////////////////////////////////////////////////////////////////
// methods

//
// Gets the number of motors supported by the owning phidget of this
// component. Use id to identify the owning phidget.
//
int phidgetCompMotor_GetCount(PhidgetId id)
{
	int iCount = 0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, iCount);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		CPhidgetMotorControl_getMotorCount(((PhidgetMotCon*) pPhidget)->handle, &iCount);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_GetCount.", phidget_GetDeviceClass(pPhidget)));
	}

	return iCount;
}

//
// Gets the current velocity of a component motor with the corresponding
// index. Use id to identify the owning phidget.
//
double phidgetCompMotor_GetVelocity(PhidgetId id, int iIndex)
{
	double dVelocity = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dVelocity);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		CPhidgetMotorControl_getVelocity(((PhidgetMotCon*) pPhidget)->handle, iIndex, &dVelocity);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_GetVelocity.", phidget_GetDeviceClass(pPhidget)));
	}

	return dVelocity;
}

//
// Sets the velocity of a component motor with the corresponding index.
// Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompMotor_SetVelocity(PhidgetId id, int iIndex, double dVelocity)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		return CPhidgetMotorControl_setVelocity(((PhidgetMotCon*) pPhidget)->handle, iIndex, dVelocity ) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_SetVelocity.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Gets the current acceleration of a component motor with the corresponding
// index. Use id to identify the owning phidget.
//
double phidgetCompMotor_GetAcceleration(PhidgetId id, int iIndex)
{
	double dAcceleration = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dAcceleration);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		CPhidgetMotorControl_getAcceleration(((PhidgetMotCon*) pPhidget)->handle, iIndex, &dAcceleration);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_GetAcceleration.", phidget_GetDeviceClass(pPhidget)));
	}

	// if we have the infinite value, reset to a standard value
	if(dAcceleration == DOUBLE_INFINITY)
		dAcceleration = phidgetCompMotor_GetAccelerationMin(id, iIndex);

	return dAcceleration;
}

//
// Sets the acceleration of a component motor with the corresponding index.
// Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompMotor_SetAcceleration(PhidgetId id, int iIndex, double dAcceleration)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		return CPhidgetMotorControl_setAcceleration(((PhidgetMotCon*) pPhidget)->handle, iIndex, dAcceleration ) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_SetAcceleration.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Gets the minimum accleration supported by a component motor with the
// corresponding index. Use id to identify the owning phidget.
//
double phidgetCompMotor_GetAccelerationMin(PhidgetId id, int iIndex)
{
	double dAcceleration = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dAcceleration);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		CPhidgetMotorControl_getAccelerationMin(((PhidgetMotCon*) pPhidget)->handle, iIndex, &dAcceleration);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_GetAccelerationMin.", phidget_GetDeviceClass(pPhidget)));
	}

	return dAcceleration;
}

//
// Gets the maximum accleration supported by a component axis with the
// corresponding index. Use id to identify the owning phidget.
//
double phidgetCompMotor_GetAccelerationMax(PhidgetId id, int iIndex)
{
	double dAcceleration = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dAcceleration);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		CPhidgetMotorControl_getAccelerationMax(((PhidgetMotCon*) pPhidget)->handle, iIndex, &dAcceleration);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_GetAccelerationMax.", phidget_GetDeviceClass(pPhidget)));
	}

	return dAcceleration;
}

//
// Gets the current current draw of a component motor with the corresponding
// index. Use id to identify the owning phidget.
//
double phidgetCompMotor_GetCurrent(PhidgetId id, int iIndex)
{
	double dCurrent = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dCurrent);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		CPhidgetMotorControl_getCurrent(((PhidgetMotCon*) pPhidget)->handle, iIndex, &dCurrent);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_GetCurrent.", phidget_GetDeviceClass(pPhidget)));
	}

	return dCurrent;
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has changed a components motor velocity. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreVelocityChange is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompMotor_SetSemaphoreIndex_VelocityChange(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		((PhidgetMotCon*) pPhidget)->iSemaphoreVelocityChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <MotCon-VelocityChange> is: %d, Phidget id = %i",
			((PhidgetMotCon*) pPhidget)->iSemaphoreVelocityChange, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_SetSemaphoreIndex_VelocityChange.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has changed a components motor current draw. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreCurrentChange is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompMotor_SetSemaphoreIndex_CurrentChange(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_MOTORCONTROL:
		((PhidgetMotCon*) pPhidget)->iSemaphoreCurrentChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <MotCon-CurrentChange> is: %d, Phidget id = %i",
			((PhidgetMotCon*) pPhidget)->iSemaphoreCurrentChange, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompMotor_SetSemaphoreIndex_CurrentChange.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}
