/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetCompServoMotor.c

Abstract:

	This file contains methods for a Phidget Component ServoMotor. 
	A component ServoMotor can belong to several phidgets e.g. the phidget Servo.
	Only if one of these phidgets is attached the component is useable.	Use
	phidget_GetDeviceClass(pointerToPhidget) to call the specific phidget methods
	in phidget21 API.

--*/

#include "PhidgetCompServoMotor.h"

//////////////////////////////////////////////////////////////////////////
// methods

//
// Gets the number of servo motors supported by the owning phidget of this
// component. Use id to identify the owning phidget.
//
int phidgetCompServoMotor_GetCount(PhidgetId id)
{
	int iCount = 0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, iCount);
	
	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_SERVO:
		CPhidgetServo_getMotorCount ( ((PhidgetServo*) pPhidget)->handle, &iCount);
		break;
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getMotorCount ( ((PhidgetAdvServo*) pPhidget)->handle, &iCount);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompServoMotor_GetCount.", phidget_GetDeviceClass(pPhidget)));
	}

	return iCount;
}

//
// Gets the current position of a component servo motor with the corresponding
// index. Use id to identify the owning phidget.
//
double phidgetCompServoMotor_GetPosition(PhidgetId id, int iIndex)
{
	double fPosition = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, fPosition);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_SERVO:
		CPhidgetServo_getPosition ( ((PhidgetServo*) pPhidget)->handle, iIndex, &fPosition);
		break;
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getPosition ( ((PhidgetAdvServo*) pPhidget)->handle, iIndex, &fPosition);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompServoMotor_GetPosition.", phidget_GetDeviceClass(pPhidget)));
	}

	// if we have the infinite value, reset to a standard value
	if(fPosition == DOUBLE_INFINITY)
		fPosition = phidgetCompServoMotor_GetPositionMin(id, iIndex);
	return fPosition;
}

//
// Sets the position of a component servo motor with the corresponding
// index. Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompServoMotor_SetPosition(PhidgetId id, int iIndex, double fPosition)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_SERVO:
		return CPhidgetServo_setPosition ( ((PhidgetServo*) pPhidget)->handle, iIndex, fPosition ) == 0;
	case PHIDCLASS_ADVANCEDSERVO:
		return CPhidgetAdvancedServo_setPosition ( ((PhidgetAdvServo*) pPhidget)->handle, iIndex, fPosition ) == 0;


	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompServoMotor_SetPosition.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Gets the maximum position supported by a component servo motor with the
// corresponding index. Use id to identify the owning phidget.
//
double phidgetCompServoMotor_GetPositionMin(PhidgetId id, int iIndex)
{
	double fPosition = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, fPosition);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_SERVO:
		CPhidgetServo_getPositionMin ( ((PhidgetServo*) pPhidget)->handle, iIndex, &fPosition);
		break;
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getPositionMin ( ((PhidgetAdvServo*) pPhidget)->handle, iIndex, &fPosition);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompServoMotor_GetPositionMin.", phidget_GetDeviceClass(pPhidget)));
	}

	return fPosition;
}

//
// Gets the minimum position supported by a component servo motor with the
// corresponding index. Use id to identify the owning phidget.
//
double phidgetCompServoMotor_GetPositionMax(PhidgetId id, int iIndex)
{
	double fPosition = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, fPosition);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_SERVO:
		CPhidgetServo_getPositionMax ( ((PhidgetServo*) pPhidget)->handle, iIndex, &fPosition);
		break;
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getPositionMax ( ((PhidgetAdvServo*) pPhidget)->handle, iIndex, &fPosition);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompServoMotor_GetPositionMax.", phidget_GetDeviceClass(pPhidget)));
	}

	return fPosition;
}

//
// Gets the current engaged state of a component servo motor with the
// corresponding index. This is true when the motor is powered
// otherwise false.
// Use id to identify the owning phidget.
//
BOOL phidgetCompServoMotor_IsEngaged(PhidgetId id, int iIndex)
{
	BOOL bState = FALSE;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, bState);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_SERVO:
		CPhidgetServo_getEngaged ( ((PhidgetServo*) pPhidget)->handle, iIndex, &bState );
		break;
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getEngaged ( ((PhidgetAdvServo*) pPhidget)->handle, iIndex, &bState );
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompServoMotor_IsEngaged.", phidget_GetDeviceClass(pPhidget)));
	}

	return bState > 0;
}

//
// Sets the engaged state of a component servo motor with the corresponding
// index. Set to true if the motor should be powered otherwise false.
// Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompServoMotor_SetEngaged(PhidgetId id, int iIndex, BOOL bState)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_SERVO:
		return CPhidgetServo_setEngaged ( ((PhidgetServo*) pPhidget)->handle, iIndex, bState ) == 0;
	case PHIDCLASS_ADVANCEDSERVO:
		return CPhidgetAdvancedServo_setEngaged ( ((PhidgetAdvServo*) pPhidget)->handle, iIndex, bState ) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompServoMotor_SetEngaged.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has changed a components servo motor position. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphorePositionChange is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompServoMotor_SetSemaphoreIndex_PositionChange(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_SERVO:
		((PhidgetServo*) pPhidget)->iSemaphorePositionChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <Servo-PositionChange> is: %d, Phidget id = %i",
			((PhidgetServo*) pPhidget)->iSemaphorePositionChange, pPhidget->id));
		return TRUE;

	case PHIDCLASS_ADVANCEDSERVO:
		((PhidgetAdvServo*) pPhidget)->iSemaphorePositionChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <AdvServo-PositionChange> is: %d, Phidget id = %i",
			((PhidgetAdvServo*) pPhidget)->iSemaphorePositionChange, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompServoMotor_SetSemaphoreIndex_PositionChange.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}
