/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetInterfaceKit.c

Abstract:

	This file contains Phidget InterfaceKit specific methods. A Phidget
	Interface Kit has a component sensor, input and output. These four
	modules - PhidgetCompAnalogSensor.c, PhidgetCompDigitalInput.c, 
	PhidgetCompDigitalOutput.c and PhidgetInterfaceKit.c - inmplement all 
	methods supplied by the phidget21 API for an InterfaceKit.

--*/

#include "PhidgetInterfaceKit.h"

//////////////////////////////////////////////////////////////////////////
// forward declarations: callback functions

int CALLBACK phidgetIfKit_inputChangeHandler(CPhidgetInterfaceKitHandle ifKitHandle, void *pUserPtr, int iIndex, int iState);
int CALLBACK phidgetIfKit_outputChangeHandler(CPhidgetInterfaceKitHandle ifKitHandle, void *pUserPtr, int iIndex, int iState);
int CALLBACK phidgetIfKit_sensorChangeHandler(CPhidgetInterfaceKitHandle ifKitHandle, void *pUserPtr, int iIndex, int iValue);

//////////////////////////////////////////////////////////////////////////
// PhidgetInterfaceKit methods

/*++

Routine Description:

	This method creates an PhidgetIfKit struct and allocates memory.
	To identify the phidgets type the deviceClass is set here.
	Additionally do common creational settings.

Arguments:

	None.

Return Value:

	Squeak VM identifier for the PhigdetId Object.

--*/
sqInt phidgetIfKit_Create()
{
	// allocate memory for a PhidgetIfKit structure
	PhidgetIfKit* pPhidget = calloc(1, sizeof(PhidgetIfKit));

	// initialise
	pPhidget->deviceClass = PHIDCLASS_INTERFACEKIT;
	phidgetIfKit_ResetSemaphores(pPhidget);

	// get id and add to global list. return the id
	return phidget_Create((Phidget*)pPhidget);
}

//
//reset semaphores
//
void phidgetIfKit_ResetSemaphores(PhidgetIfKit *pPhidget)
{
	if(pPhidget != NULL)
	{
		phidget_ResetSemaphores((Phidget*)pPhidget);

		pPhidget->iSemaphoreSensorChange = 0;
		pPhidget->iSemaphoreInputChange = 0;
		pPhidget->iSemaphoreOutputChange = 0;
	}
	else
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetIfKit_ResetSemaphores for pPhidget."));
}

/*++

Routine Description:

	This method is called while opening a phidget. The actual specific
	phidget object will be created and the specific callbacks will be
	registered.

Arguments:

	pPhidget - pointer of the Phidget to open the hardware connection.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetIfKit_Open(PhidgetIfKit* pPhidget)
{
	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetIfKit_Open for pPhidget."));
		return FALSE;
	}

	//create the InterfaceKit object
	pPhidget->handle = 0;
	CPhidgetInterfaceKit_create(&(pPhidget->handle));

	//Registers the callbacks that will run if an input/ouput changes or if the sensor value changes by more than the OnSensorChange trigger.
	//Requires the handle for the Phidget, the function that will be called, and an arbitrary pointer that will be supplied to the callback function (may be NULL).
	CPhidgetInterfaceKit_set_OnInputChange_Handler (pPhidget->handle, phidgetIfKit_inputChangeHandler, (void *)pPhidget->id);
	CPhidgetInterfaceKit_set_OnSensorChange_Handler (pPhidget->handle, phidgetIfKit_sensorChangeHandler, (void *)pPhidget->id);
	CPhidgetInterfaceKit_set_OnOutputChange_Handler (pPhidget->handle, phidgetIfKit_outputChangeHandler, (void *)pPhidget->id);

	// do phidget common open operations
	phidget_Open_Base((Phidget*)pPhidget);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// callback functions

/*++

Routine Description:

	This callback is called when an input of a Phidget InterfaceKit with
	the id pUserPtr has changed.
	It signals the semaphore iSemaphoreInputChange.

Arguments:

	ifKitHandle - handle of the InterfaceKit to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the input that generated the event

	iState - the input state (on or off)

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetIfKit_inputChangeHandler(CPhidgetInterfaceKitHandle ifKitHandle, void *pUserPtr, int iIndex, int iState)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetIfKit* pPhidget = (PhidgetIfKit*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<IfKit-InputChange> id = %i, semaphore = %i, index = %i, state = %i",
		pPhidget->id, pPhidget->iSemaphoreInputChange, iIndex, iState));
	
	if (pPhidget->iSemaphoreInputChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreInputChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <IfKit-InputChange>, id = %i", pPhidget->id));
	return 0;
}

/*++

Routine Description:

	This callback is called when an output of a Phidget InterfaceKit with
	the id pUserPtr has changed.
	It signals the semaphore iSemaphoreOutputChange.

Arguments:

	ifKitHandle - handle of the InterfaceKit to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the output that generated the event

	iState - the output state (on or off)

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetIfKit_outputChangeHandler(CPhidgetInterfaceKitHandle ifKitHandle, void *pUserPtr, int iIndex, int iState)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetIfKit* pPhidget = (PhidgetIfKit*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<IfKit-OutputChange> id = %i, semaphore = %i, index = %i, state = %i",
		pPhidget->id, pPhidget->iSemaphoreOutputChange, iIndex, iState));

	if (pPhidget->iSemaphoreOutputChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreOutputChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <IfKit-OutputChange>, id = %i", pPhidget->id));
	return 0;
}

/*++

Routine Description:

	This callback is called when a sensor of a Phidget InterfaceKit with
	the id pUserPtr is changed by more than OnSensorChange trigger.
	It signals the semaphore iSemaphoreSensorChange.

Arguments:

	ifKitHandle - handle of the InterfaceKit to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the sensor that generated the event

	iValue - the sensor read value

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetIfKit_sensorChangeHandler(CPhidgetInterfaceKitHandle ifKitHandle, void *pUserPtr, int iIndex, int iValue)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetIfKit* pPhidget = (PhidgetIfKit*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<IfKit-SensorChange> id = %i, semaphore = %i, index = %i, value = %i",
		pPhidget->id, pPhidget->iSemaphoreSensorChange, iIndex, iValue));

	if (pPhidget->iSemaphoreSensorChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreSensorChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <IfKit-SensorChange>, id = %i", pPhidget->id));
	return 0;
}
