/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetMotorControl.c

Abstract:

	This file contains Phidget Motor Control specific methods. A Phidget
	Motor Control has a component motor and input. These three modules - 
	PhidgetCompMotor, PhidgetCompDigitalInput.c and PhidgetMotorControl.c
	- inmplement all methods supplied by the phidget21 API for an Motor 
	Control.

--*/

#include "PhidgetMotorControl.h"

//////////////////////////////////////////////////////////////////////////
// forward declarations: callback functions

int CALLBACK phidgetMotCon_inputChangeHandler(CPhidgetMotorControlHandle motConHandle, void *pUserPtr, int iIndex, int iState);
int CALLBACK phidgetMotCon_currentChangeHandler(CPhidgetMotorControlHandle motConHandle, void *pUserPtr, int iIndex, double dCurrent);
int CALLBACK phidgetMotCon_velocityChangeHandler(CPhidgetMotorControlHandle motConHandle, void *pUserPtr, int iIndex, double dVelocity);

//////////////////////////////////////////////////////////////////////////
// PhidgetMotorControl methods

/*++

Routine Description:

	This method creates an PhidgetMotCon struct and allocates memory.
	To identify the phidgets type the deviceClass is set here.
	Additionally do common creational settings.

Arguments:

	None.

Return Value:

	Squeak VM identifier for the PhigdetId Object.

--*/
sqInt phidgetMotCon_Create()
{
	// allocate memory for a Motor Control structure
	PhidgetMotCon* pPhidget = calloc(1, sizeof(PhidgetMotCon));

	// initialise
	pPhidget->deviceClass = PHIDCLASS_MOTORCONTROL;
	phidgetMotCon_ResetSemaphores(pPhidget);

	// get id and add to global list. return the id
	return phidget_Create((Phidget*)pPhidget);
}

//
//reset semaphores
//
void phidgetMotCon_ResetSemaphores(PhidgetMotCon *pPhidget)
{
	if(pPhidget != NULL)
	{
		phidget_ResetSemaphores((Phidget*)pPhidget);

		pPhidget->iSemaphoreInputChange = 0;
		pPhidget->iSemaphoreVelocityChange = 0;
		pPhidget->iSemaphoreCurrentChange = 0;
	}
	else
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetMotCon_ResetSemaphores for pPhidget."));
}


/*++

Routine Description:

	This method is called while opening a phidget. The actuall specific
	phidget object will be created and the specific callbacks will be
	registered.

Arguments:

	pPhidget - pointer of the Phidget to open the hardware connection.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetMotCon_Open(PhidgetMotCon* pPhidget)
{
	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetMotCon_Open for pPhidget."));
		return FALSE;
	}

	//create the Motor Control object
	pPhidget->handle = 0;
	CPhidgetMotorControl_create(&(pPhidget->handle));

	//Registers the callbacks that will run if an input, the current draw (circuit) or a velocity changes
	//Requires the handle for the Phidget, the function that will be called, and an arbitrary pointer that will be supplied to the callback function (may be NULL).
	CPhidgetMotorControl_set_OnInputChange_Handler (pPhidget->handle, phidgetMotCon_inputChangeHandler,(void *)pPhidget->id);
	CPhidgetMotorControl_set_OnVelocityChange_Handler (pPhidget->handle, phidgetMotCon_velocityChangeHandler,(void *)pPhidget->id);
	CPhidgetMotorControl_set_OnCurrentChange_Handler (pPhidget->handle, phidgetMotCon_currentChangeHandler,(void *)pPhidget->id);
	
	// do phidget common open operations
	phidget_Open_Base((Phidget*)pPhidget);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// callback functions

/*++

Routine Description:

	This callback is called when an input of a Phidget Motor Control with
	the id pUserPtr has changed.
	It signals the semaphore iSemaphoreInputChange.

Arguments:

	motConHandle - handle of the Motor Control to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the input that generated the event

	iState - the input state (on or off)

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetMotCon_inputChangeHandler(CPhidgetMotorControlHandle motConHandle, void *pUserPtr, int iIndex, int iState)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetMotCon* pPhidget = (PhidgetMotCon*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<MotCon-InputChange> id = %i, semaphore = %i, index = %i, value = %i",
		pPhidget->id, pPhidget->iSemaphoreInputChange, iIndex, iState));
	
	if (pPhidget->iSemaphoreInputChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreInputChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <MotCon-InputChange>, id = %i", pPhidget->id));
	return 0;
}

/*++

Routine Description:

	This callback is called when the current draw of a motor of a Phidget
	Motor Control with the id pUserPtr has changed.
	It signals the semaphore iSemaphoreCurrentChange.

Arguments:

	motConHandle - handle of the Motor Control to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the motor that generated the event

	dCurrent - the motor read current draw

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetMotCon_currentChangeHandler(CPhidgetMotorControlHandle motConHandle, void *pUserPtr, int iIndex, double dCurrent)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetMotCon* pPhidget = (PhidgetMotCon*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<MotCon-CurrentChange> id = %i, semaphore = %i, index = %i, value = %f",
		pPhidget->id, pPhidget->iSemaphoreCurrentChange, iIndex, dCurrent));
	
	if (pPhidget->iSemaphoreCurrentChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreCurrentChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <MotCon-CurrentChange>, id = %i", pPhidget->id));
	return 0;
}

/*++

Routine Description:

	This callback is called when a motor of a Phidget Motor Control with 
	the id pUserPtr	has changed its velocity.
	It signals the semaphore iSemaphoreVelocityChange.

Arguments:

	motConHandle - handle of the Motor Control to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the motor that generated the event

	dVelocity - the motor read velocity

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetMotCon_velocityChangeHandler(CPhidgetMotorControlHandle motConHandle, void *pUserPtr, int iIndex, double dVelocity)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetMotCon* pPhidget = (PhidgetMotCon*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<MotCon-VelocityChange> id = %i, semaphore = %i, index = %i, value = %f",
		pPhidget->id, pPhidget->iSemaphoreVelocityChange, iIndex, dVelocity));
	
	if (pPhidget->iSemaphoreVelocityChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreVelocityChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <MotCon-VelocityChange>, id = %i", pPhidget->id));
	return 0;
}
